package gs

import sbt._
import sbt.Keys._

object GsPlugin extends AutoPlugin {
  override def trigger = allRequirements

  val autoImport = GsKeys

  import autoImport._

  /**
    * Prefers to load credentials from file, if a file is available. Otherwise
    * mandates that the following environment variables exist:
    *
    *   - `GS_MAVEN_USER`
    *   - `GS_MAVEN_TOKEN`
    *
    * @return The selected credentials for GS Maven.
    */
  def selectCredentials(): Credentials =
    if ((Path.userHome / ".sbt" / ".credentials").exists())
      Credentials(Path.userHome / ".sbt" / ".credentials")
    else
      Credentials.apply(
        realm =Gs.MavenRealm,
        host = Gs.MavenHost,
        userName = sys.env
          .get(Gs.Environment.MavenUser)
          .getOrElse(
            throw new RuntimeException(
              s"You must either provide ~/.sbt/.credentials or specify the ${Gs.Environment.MavenUser} environment variable."
            )
          ),
        passwd = sys.env
          .get(Gs.Environment.MavenToken)
          .getOrElse(
            throw new RuntimeException(
              s"You must either provide ~/.sbt/.credentials or specify the ${Gs.Environment.MavenToken} environment variable."
            )
          )
        )

  override def projectSettings: Seq[Setting[_]] = Seq(
    credentials += selectCredentials(),
    publishMavenStyle := true,
    Test / Keys.publishArtifact := false,
    pomIncludeRepository := Function.const(false),
    scmInfo := Some(
      ScmInfo(
        browseUrl = url(Gs.gitRepo(gsProjectName.value)), 
        connection = Gs.gitSsh(gsProjectName.value)
      )
    ),
    licenses := List(Gs.Apache2),
    homepage := Some(url(Gs.gitRepo(gsProjectName.value))),
    organizationName := Gs.OrganizationName,
    organization := Gs.GroupId,
    organizationHomepage := Some(url("https://garrity.co/")),
    publishTo := {
      val repo = "https://maven.garrity.co/"
      if (Keys.version.value.endsWith("SNAPSHOT"))
        Some("Garrity Software Maven" at repo + "snapshots")
      else
        Some("Garrity Software Maven" at repo + "gs")
    },
    scalacOptions := {
      val prev = Keys.scalacOptions.value
      prev ++ Gs.CompilerOptions
    }
  )

}
